# ZKDF

The ZKDF [key derivation function](https://en.wikipedia.org/wiki/Key_derivation_function) works by filling out a large *garble buffer* (256 KiB) with noisy data based on a user supplied passphrase and a 256-bit salt, and then calculate the SHA2-256 hash of the garble buffer. More specifically:

1. Initialize state:
    1. Initialize the state of the PRNG ([xoshiro256\*\*](https://prng.di.unimi.it/)) with the 256-bit salt value.
    2. Fill the garble buffer with zeroes.
2. Iterate *N* times:
    1. **Garble phase**: XOR the contents of the garble buffer with the (repeated) *passphrase* and a random number sequence that is derieved from a combination of the PRNG and non-reversible, sequentially dependent 128-bit integer arithmetic.
    2. **Shuffle phase**: Move around and mutate the bytes of the garble buffer in a random access pattern.
3. Calculate the SHA2-256 hash of the garble buffer.

## Technical properties of ZKDF

* Can handle very long passphrases (up to 262,143 bytes, UTF-8 encoded).
* The function has an [avalanche effect](https://en.wikipedia.org/wiki/Avalanche_effect) (even after a single pass):
    - Changing a single bit in the passphrase changes ~50% of the bits in the garble buffer.
    - Changing a single bit in the salt changes ~50% of the bits in the garble buffer.
* The read/write access pattern in the *shuffle phase* has a uniform distribution (e.g. the access frequencies of all elements in the buffer are the same).
* The final [SHA2-256](https://en.wikipedia.org/wiki/SHA-2) hash step reduces the contents of the garble buffer to a 256-bit key, and adds the extra security of a proven cryptographic hash function.

## Speed

As a rule of thumb, most laptop, desktop and server class CPUs launched after 2010 can perform 50-80 ZKDF passes/s per GHz on a single core.

**Note:** ZKDF is designed to require sequential processing, so having several CPU cores does not speed up the generation of a single key. However, for the purpose of [brute-forcing](https://en.wikipedia.org/wiki/Brute-force_attack) a passphrase it helps to have multiple CPU cores (each core can guess one passphrase independently of the other cores).

The number of ZKDF passes that can be performed per second is listed for a few different CPUs in the following table:

| CPU | GHz | Year | Class | Passes/s (1 core) |
| --- | --- | --- | --- | --- |
| Intel i7-1270P | 4.7 | 2022 | Laptop | 406 |
| AMD Ryzen 9 3950X | 4.2 | 2019 | Desktop | 314 |
| AMD EPYC 9224 | 3.7 | 2022 | Server | 307 |
| AMD Ryzen 9 3900X | 3.7 | 2019 | Desktop | 274 |
| Intel Xeon Gold 5218 | 3.6 | 2019 | Server | 248 |
| AMD EPYC 7402P | 3.3 | 2019 | Server | 237 |
| Intel Xeon Gold 5118 | 3.2 | 2017 | Server | 220 |
| Intel i7-3615QM | 3.3 | 2012 | Laptop | 186 |
| Intel i5-4200M | 3.1 | 2013 | Laptop | 171 |
| Intel Celeron N4500 | 2.8 | 2021 | Laptop | 129 |
| ARM Cortex-A72 (BCM2711) | 1.5 | 2019 | Embedded | 38 |
| AMD E2-1800 | 1.7 | 2012 | Laptop | 33 |
| ARM Cortex-A53 (BCM2837) | 1.2 | 2016 | Embedded | 19 |

