#!/usr/bin/env bash
# -*- mode: bash; tab-width: 4; indent-tabs-mode: nil; -*-

set -o errexit

PROJ_ROOT=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )"/.. &> /dev/null && pwd)

BUILD_ROOT=${PROJ_ROOT}/out
if [ "$#" -eq 1 ]; then
    BUILD_ROOT=$1
fi

mkdir -p "${BUILD_ROOT}"
cd "${BUILD_ROOT}"

###############################################################################
# Linux tests
###############################################################################
mkdir -p "${BUILD_ROOT}/linux"

echo "########################################################################"
echo "# Run CMake"
echo "########################################################################"
cmake -G Ninja \
      -DCMAKE_BUILD_TYPE=Release \
      -S "${PROJ_ROOT}/src" \
      -B "${BUILD_ROOT}/linux"
echo "⇨ Success 🎉"
echo

echo "########################################################################"
echo "# Build"
echo "########################################################################"
ninja -C "${BUILD_ROOT}/linux"
echo "⇨ Success 😺"
echo

echo "########################################################################"
echo "# Run unit tests"
echo "########################################################################"
CTEST_OUTPUT_ON_FAILURE=ON ninja -C "${BUILD_ROOT}/linux" test
echo "⇨ Success 👏"
echo

echo "########################################################################"
echo "# Check code formatting"
echo "########################################################################"
"${PROJ_ROOT}"/scripts/format.sh --check
echo "⇨ Success 💪"
echo

echo "########################################################################"
echo "# Run linting"
echo "########################################################################"
"${PROJ_ROOT}"/scripts/lint.sh "${BUILD_ROOT}/linux"
echo "⇨ Success 🥳"
echo

echo "########################################################################"
echo "# Run benchmarks"
echo "########################################################################"
for BENCH in $(find "${BUILD_ROOT}/linux" -name 'benchmark_*' -type f -perm /100 | sort); do
    "${BENCH}"
    echo
done
echo "⇨ Success 🚀"
echo

# Check code coverage
# TODO: Implement me.

###############################################################################
# Windows tests
###############################################################################
mkdir -p "${BUILD_ROOT}/win"

echo "########################################################################"
echo "# Run CMake - Windows (MinGW)"
echo "########################################################################"
cmake -G Ninja \
      -DCMAKE_TOOLCHAIN_FILE="${PROJ_ROOT}/src/mingw-cross-toolchain.cmake" \
      -DCMAKE_BUILD_TYPE=Release \
      -S "${PROJ_ROOT}/src" \
      -B "${BUILD_ROOT}/win"
echo "⇨ Success 🤩"
echo

echo "########################################################################"
echo "# Build - Windows"
echo "########################################################################"
ninja -C "${BUILD_ROOT}/win"
echo "⇨ Success 🌟"
echo

echo "⇨ All tests passed! 💯"

