#!/bin/bash
# -*- mode: bash; tab-width: 4; indent-tabs-mode: nil; -*-

set -o errexit

PROJ_ROOT=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )"/.. &> /dev/null && pwd)
BUILD_DIR=${PWD}

GCOV=gcov
LCOV=lcov

# Ensure that we are using a correct build folder.
if [ ! -f "${BUILD_DIR}/CMakeCache.txt" ] ; then
    echo "***Error: This script should be run from a CMake build folder."
    exit 1
fi

# Remove any old coverage files.
OLD_GCOV_FILES=$(find "${BUILD_DIR}" -regex '.*\.[ch]\.gc.*')
if [ -n "${OLD_GCOV_FILES}" ] ; then
    echo "Deleting old gcov files"
    rm -f ${OLD_GCOV_FILES}
fi

# Build the tests.
cmake --build "${BUILD_DIR}" --target clean
cmake --build "${BUILD_DIR}"

# Run the tests.
cmake --build "${BUILD_DIR}" --target test

# Run gcov.
GCDAS=$(find "${BUILD_DIR}" -regex '.*\.c\.gcda')
if [ -z "${GCDAS}" ] ; then
    echo "***Error: No coverage files found. Please re-build with COVERAGE=ON."
    exit 1
fi
"${GCOV}" -b -f ${GCDAS}

# Run lcov.
"${LCOV}" -c                                                                  \
          --directory "${BUILD_DIR}"                                          \
          --output-file "${BUILD_DIR}/lcov_stats_all.info"
"${LCOV}" --remove "${BUILD_DIR}/lcov_stats_all.info"                         \
          --output-file "${BUILD_DIR}/lcov_stats.info"                        \
          '/usr/*'                                                            \
          '*/error.c'                                                         \
          '*/test/*'

# Generate HTML report.
rm -rf "${BUILD_DIR}/html"
mkdir -p "${BUILD_DIR}/html"
genhtml --ignore-errors source "${BUILD_DIR}/lcov_stats.info"                 \
        --output-directory="${BUILD_DIR}/html"

# Launch web browser to view the report.
xdg-open "${BUILD_DIR}/html/index.html"

