#!/usr/bin/env bash
# -*- mode: bash; tab-width: 4; indent-tabs-mode: nil; -*-

set -o errexit

PROJ_ROOT=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )"/.. &> /dev/null && pwd)

function help {
    echo "Usage: $0 [options]"
    echo ""
    echo "Options:"
    echo "  -c, --check    Only check formatting and set exit code"
    echo "  -h, --help     Show this text"
}

CHECK=no
for arg in "$@" ; do
    case $arg in
        -h|--help)
            help
            exit 0
            ;;
        -c|--check)
            CHECK=yes
            ;;
        *)
            echo "*** Invalid argument: $arg"
            help
            exit 1
            ;;
    esac
done

echo "[ clang-format ]"
if command -v clang-format &> /dev/null
then
    SRCS=$(find "${PROJ_ROOT}/src" -regex '.*\.\(c\|h\)')
    if [ "$CHECK" == "yes" ] ; then
        clang-format --dry-run --Werror ${SRCS}
    else
        clang-format -i ${SRCS}
    fi
else
    echo "***Warning: clang-format not found"
fi

echo "[ cmake-format ]"
if command -v cmake-format &> /dev/null
then
    SRCS=$(find "${PROJ_ROOT}/src" -regex '\(.*CMakeLists\.txt\|.*\.cmake\)')
    if [ "$CHECK" == "yes" ] ; then
        cmake-format --check ${SRCS}
    else
        cmake-format -i ${SRCS}
    fi
else
    echo "***Warning: cmake-format not found"
fi
