// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// An implementation of the AES cipher.
// Based on the public domain implementation tiny-AES by "kokke"".
// See: https://github.com/kokke/tiny-AES-c
//------------------------------------------------------------------------------

#ifndef AES_H_
#define AES_H_

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

// Configuration for AES-256.
#define AES_BLOCK_BYTES 16
#define AES_IV_BYTES AES_BLOCK_BYTES
#define AES_KEY_BYTES 32
#define AES_ROUND_KEY_BYTES 240

typedef struct {
  uint8_t round_key[AES_ROUND_KEY_BYTES];
  uint8_t iv[AES_IV_BYTES];
  int method;  // Used for selecting acceleration method.
} aes256_ctx_t;

// Used for testing: The force_generic flag forces the implementation to use
// non-accelerated code paths.
void aes_256_init_ex(aes256_ctx_t* ctx,
                     const uint8_t key[AES_KEY_BYTES],
                     const uint8_t iv[AES_IV_BYTES],
                     bool force_generic);

static inline void aes_256_init(aes256_ctx_t* ctx,
                                const uint8_t key[AES_KEY_BYTES],
                                const uint8_t iv[AES_IV_BYTES]) {
  aes_256_init_ex(ctx, key, iv, false);
}
void aes_256_cbc_encrypt_buffer(aes256_ctx_t* ctx, uint8_t* buf, size_t length);
void aes_256_cbc_decrypt_buffer(aes256_ctx_t* ctx, uint8_t* buf, size_t length);

#endif  // AES_H_
