// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#ifndef POLY1305_H_
#define POLY1305_H_

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

// The number of bytes for the key material for the Poly1305 algorithm.
#define POLY1305_KEY_BYTES 32

// The number of bytes required by a Poly1305 hash (128 bits = 16 bytes).
#define POLY1305_HASH_BYTES 16

// The number of bytes per Poly1305 block.
#define POLY1305_BLOCK_BYTES 16

typedef struct {
  uint32_t r[5];
  uint32_t h[5];
  uint32_t s[4];
  uint8_t buffer[POLY1305_BLOCK_BYTES];
  size_t leftover;
  bool final;
} poly1305_ctx_t;

// Initialize a Poly1305 context (start fresh).
void poly1305_init(poly1305_ctx_t* ctx, const uint8_t key[POLY1305_KEY_BYTES]);

// Append & hash more data.
void poly1305_append(poly1305_ctx_t* ctx,
                     const uint8_t* message,
                     size_t length);

// Finalize the data and return the resulting hash.
void poly1305_finalize(poly1305_ctx_t* ctx, uint8_t hash[POLY1305_HASH_BYTES]);

// Convenience function for hashing a single in-memory buffer.
void poly1305(const uint8_t key[POLY1305_KEY_BYTES],
              const uint8_t* message,
              size_t length,
              uint8_t hash[POLY1305_HASH_BYTES]);

#endif  // POLY1305_H_
