// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// An implementation of the Serpent cipher.
//
// Based on the public domain C++ implementation by Wei Dai:
//  - https://github.com/weidai11/cryptopp
// ...and the reference C implementation by Frank Stajano:
//  - https://www.cl.cam.ac.uk/~rja14/serpent.html
//------------------------------------------------------------------------------

#ifndef SERPENT_H_
#define SERPENT_H_

#include <stddef.h>
#include <stdint.h>

// The number of bytes for a Serpent-256 key (256 bits = 32 bytes).
#define SERPENT_KEY_BYTES 32

// The number of bytes for a Serpent-256 block (128 bits = 16 bytes).
#define SERPENT_BLOCK_BYTES 16

// The number of bytes for a Serpent-256 IV (128 bits = 16 bytes).
#define SERPENT_IV_BYTES SERPENT_BLOCK_BYTES

typedef struct {
  uint32_t expkey[33 * 4];  // Series of subkeys.
  uint8_t iv[SERPENT_IV_BYTES];
} serpent256_ctx_t;

void serpent_256_init(serpent256_ctx_t* ctx,
                      const uint8_t key[SERPENT_KEY_BYTES],
                      const uint8_t iv[SERPENT_IV_BYTES]);

void serpent_256_cbc_encrypt_buffer(serpent256_ctx_t* ctx,
                                    uint8_t* buf,
                                    size_t length);

void serpent_256_cbc_decrypt_buffer(serpent256_ctx_t* ctx,
                                    uint8_t* buf,
                                    size_t length);

#endif  // SERPENT_H_
