// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// An implementation of the SHA2 256-bit hashing algorithm.
//
// Based on pseudocode from Wikipedia:
// https://en.wikipedia.org/wiki/SHA-2#Pseudocode
//------------------------------------------------------------------------------

#ifndef SHA2_H_
#define SHA2_H_

#include <stddef.h>
#include <stdint.h>

// The number of bytes required by a SHA-2 256-bit hash (256 bits = 32 bytes).
#define SHA256_HASH_BYTES 32

// The number of bytes per hash chunk.
#define SHA256_CHUNK_BYTES 64

typedef struct {
  uint32_t h[8];
  size_t total_length;
  uint8_t buffer[SHA256_CHUNK_BYTES];
} sha2_256_ctx_t;

// Initialize a SHA2-256 context (start fresh).
void sha2_256_init(sha2_256_ctx_t* ctx);

// Append & hash more data.
void sha2_256_append(sha2_256_ctx_t* ctx, const uint8_t* msg, size_t length);

// Finalize the data and return the resulting hash.
void sha2_256_finalize(sha2_256_ctx_t* ctx, uint8_t hash[SHA256_HASH_BYTES]);

// Convenience function for hashing a single in-memory buffer.
void sha2_256(const uint8_t* msg,
              size_t length,
              uint8_t hash[SHA256_HASH_BYTES]);

#endif  // SHA2_H_
