// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Twofish cipher implementation.
// Based on a public domain implementation by Drew Csillag.
// See: https://www.schneier.com/academic/twofish/download/
//------------------------------------------------------------------------------

#ifndef TWOFISH_H_
#define TWOFISH_H_

#include <stddef.h>
#include <stdint.h>

// The number of bytes for a Twofish-256 key (256 bits = 32 bytes).
#define TWOFISH_KEY_BYTES 32

// The number of bytes for a Twofish block (128 bits = 16 bytes).
#define TWOFISH_BLOCK_BYTES 16

// The number of bytes for a Twofish IV (same as block length).
#define TWOFISH_IV_BYTES TWOFISH_BLOCK_BYTES

typedef struct {
  uint32_t QF[4][256];
  uint32_t K[40];
  uint8_t iv[TWOFISH_IV_BYTES];
} twofish256_ctx_t;

void twofish_256_init(twofish256_ctx_t* ctx,
                      const uint8_t key[TWOFISH_KEY_BYTES],
                      const uint8_t iv[TWOFISH_IV_BYTES]);
void twofish_256_cbc_encrypt_buffer(twofish256_ctx_t* ctx,
                                    uint8_t* buf,
                                    size_t length);
void twofish_256_cbc_decrypt_buffer(twofish256_ctx_t* ctx,
                                    uint8_t* buf,
                                    size_t length);

#endif  // TWOFISH_H_
