// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// A (fairly) portable function for querying CPUID on x86 CPU:s.
//------------------------------------------------------------------------------

#ifndef X86CPUID_H_
#define X86CPUID_H_

// We prefer a single define, regardless of compiler.
#if (!defined(__x86_64__)) && (defined(_M_AMD64) || defined(_M_X64) || \
                               defined(__amd64__) || defined(__amd64))
#define __x86_64__
#elif (!defined(__i386__)) &&                                   \
    (defined(__i386) || defined(_M_IX86) || defined(__X86__) || \
     defined(_X86_) || defined(__I86__))
#define __i386__
#endif

// Are we compiling for x86 at all?
#if defined(__x86_64__) || defined(__i386__)

#include <stdint.h>

#if defined(__GNUC__)
#include <cpuid.h>
#endif
#if defined(_MSC_VER)
#include <intrin.h>
#endif

typedef struct {
  uint32_t eax;
  uint32_t ebx;
  uint32_t ecx;
  uint32_t edx;
} cpuid_regs_t;

static inline void x86cpuid(const uint32_t func,
                            const uint32_t subfunc,
                            cpuid_regs_t* regs) {
#if defined(_MSC_VER)
  int cpu_info[4] = {-1};
  __cpuidex(cpu_info, (int)func, (int)subfunc);
  regs->eax = cpu_info[0];
  regs->ebx = cpu_info[1];
  regs->ecx = cpu_info[2];
  regs->edx = cpu_info[3];
#elif defined(__GNUC__) && defined(__cpuid_count)
  __cpuid_count(func, subfunc, regs->eax, regs->ebx, regs->ecx, regs->edx);
#else
  __asm__ __volatile__(
#if defined(__x86_64__)
      "pushq %%rbx;"
#else
      "pushl %%ebx;"
#endif
      "cpuid;"
      "movl %%ebx, %[reg_ebx];"
#if defined(__x86_64__)
      "popq %%rbx;"
#else
      "popl %%ebx;"
#endif
      : "=a"(regs->eax),
        [reg_ebx] "=rm"(regs->ebx),
        "=c"(regs->ecx),
        "=d"(regs->edx)
      : "a"(func), "c"(subfunc)
      : "cc");
#endif
}

#endif  // __x86_64__ || __i386__

#endif  // X86CPUID_H_
