// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "xoshiro.h"

#include "../error.h"
#include <stddef.h>

void xoshiro256_init(const uint8_t seed[XOSHIRO256_BYTES],
                     xoshiro256_state_t* state) {
  PRECONDITION(seed != NULL);
  PRECONDITION(state != NULL);

  uint64_t x_ored = 0U;
  for (int i = 0; i < 4; ++i) {
    // Extract 64 bits from the seed buffer (read as little endian).
    uint64_t x = (((uint64_t)seed[(i * 8) + 7]) << 56) |
                 (((uint64_t)seed[(i * 8) + 6]) << 48) |
                 (((uint64_t)seed[(i * 8) + 5]) << 40) |
                 (((uint64_t)seed[(i * 8) + 4]) << 32) |
                 (((uint64_t)seed[(i * 8) + 3]) << 24) |
                 (((uint64_t)seed[(i * 8) + 2]) << 16) |
                 (((uint64_t)seed[(i * 8) + 1]) << 8) |
                 ((uint64_t)seed[(i * 8) + 0]);

    // Fill out the next 64-bit word in the xoshiro state.
    state->s[i] = x;

    // Keep track of whether or not all bits are zero.
    x_ored |= x;
  }

  // Special case: All-zero is not a valid state.
  if (x_ored == 0U) {
    state->s[0] = 0x12345678U;
  }
}
