// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// An implementation of the xoshiro256** pseudorandom number generator.
//
// See: https://prng.di.unimi.it/xoshiro256starstar.c
//------------------------------------------------------------------------------

#ifndef XOSHIRO_H_
#define XOSHIRO_H_

#include <stdint.h>

// The number of bytes of the state (256 bits = 32 bytes).
#define XOSHIRO256_BYTES 32

typedef struct {
  uint64_t s[4];
} xoshiro256_state_t;

// Initialize an xoshiro256_state_t buffer from a data buffer.
void xoshiro256_init(const uint8_t seed[XOSHIRO256_BYTES],
                     xoshiro256_state_t* state);

static inline uint64_t xoshiro_rotl_(const uint64_t x, const unsigned k) {
  return (x << k) | (x >> (64 - k));
}

// Perform one step of the xoshiro256** algorithm.
// Returns a 64-bit word.
static inline uint64_t xoshiro256ss(xoshiro256_state_t* state) {
  const uint64_t result = xoshiro_rotl_(state->s[1] * 5, 7U) * 9;
  const uint64_t t = state->s[1] << 17;
  state->s[2] ^= state->s[0];
  state->s[3] ^= state->s[1];
  state->s[1] ^= state->s[2];
  state->s[0] ^= state->s[3];
  state->s[2] ^= t;
  state->s[3] = xoshiro_rotl_(state->s[3], 45U);
  return result;
}

#endif  // XOSHIRO_H_
