// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// This is ZKDF - A compute intensive key derivation function.
//
// ZKDF converts a passhprase and a salt into an encryption key suitable for
// 256-bit key ciphers (such as AES-256 or ChaCha20-256).
//------------------------------------------------------------------------------

#ifndef ZKDF_H_
#define ZKDF_H_

#include <stdint.h>

// The number of bytes of a key (256 bits = 32 bytes).
#define ZKDF_KEY_BYTES 32

// The number of bytes of a salt.
// Note: This must match the size (in bytes) of the xoshiro256 state.
#define ZKDF_SALT_BYTES 32

void zkdf_make_key(const char* passphrase,
                   const uint8_t salt[ZKDF_SALT_BYTES],
                   int num_passes,
                   uint8_t key[ZKDF_KEY_BYTES]);

#endif  // ZKDF_H_
