// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "error.h"

#include "stringutils.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_SHORT_NAME 100

void print_precondition_failed_(const char* file, int line) {
  // Extract the file name part (drop the directory part).
#if defined(_WIN32)
  const char DIR_SEPARATOR = '\\';
#else
  const char DIR_SEPARATOR = '/';
#endif
  const char* last_slash = strrchr(file, DIR_SEPARATOR);
  if (last_slash != NULL) {
    file = last_slash + 1;
  }

  // Drop the file extension.
  const char* short_name = file;
#if defined(NDEBUG)
  char short_name_buf[MAX_SHORT_NAME];
  if (string_copy(file, short_name_buf, sizeof(short_name_buf))) {
    char* first_dot = strchr(short_name_buf, '.');
    if (first_dot != NULL) {
      *first_dot = 0;
    }
    short_name = short_name_buf;
  }
#endif

  // Print the error message.
  fflush(stdout);
#if defined(NDEBUG)
  (void)line;
  fprintf(stderr, "***Error: %s: Precondition failed\n", short_name);
#else
  fprintf(stderr, "***Error: %s:%d: Precondition failed\n", short_name, line);
#endif
}
