// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "passphrase_common.h"

#include "stringutils.h"
#include <signal.h>
#include <stdbool.h>
#include <stdlib.h>

#if defined(__GNUC__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#endif  // __GNUC__
// NOLINTNEXTLINE(misc-header-include-cycle)
#include <gtk/gtk.h>
#include <limits.h>
#if defined(__GNUC__)
#pragma GCC diagnostic pop
#endif  // __GNUC__
#include <dlfcn.h>
#include <sys/wait.h>

//------------------------------------------------------------------------------
// GTK function pointers.
//------------------------------------------------------------------------------

static struct {
  void* handle;
  GType (*box_get_type)(void);
  GtkWidget* (*box_new)(GtkOrientation, gint);
  void (*box_pack_start)(GtkBox*, GtkWidget*, gboolean, gboolean, guint);
  void (*box_set_spacing)(GtkBox*, gint);
  void (*container_add)(GtkContainer*, GtkWidget*);
  GType (*container_get_type)(void);
  void (*container_set_border_width)(GtkContainer*, guint);
  GtkWidget* (*dialog_get_content_area)(GtkDialog*);
  GType (*dialog_get_type)(void);
  GtkWidget* (*dialog_new_with_buttons)(const gchar*,
                                        GtkWindow*,
                                        GtkDialogFlags,
                                        const gchar*,
                                        ...);
  gint (*dialog_run)(GtkDialog*);
  void (*dialog_set_default_response)(GtkDialog*, gint);
  const gchar* (*entry_get_text)(GtkEntry*);
  GType (*entry_get_type)(void);
  GtkWidget* (*entry_new)(void);
  void (*entry_set_activates_default)(GtkEntry*, gboolean);
  void (*entry_set_input_purpose)(GtkEntry*, GtkInputPurpose);
  void (*entry_set_placeholder_text)(GtkEntry*, const gchar*);
  void (*entry_set_visibility)(GtkEntry*, gboolean);
  void (*entry_set_width_chars)(GtkEntry*, gint);
  gboolean (*events_pending)(void);
  GtkWidget* (*image_new_from_icon_name)(const gchar*, GtkIconSize);
  gboolean (*init_check)(int*, char***);
  GtkWidget* (*label_new)(const gchar*);
  gboolean (*main_iteration)(void);
  void (*widget_destroy)(GtkWidget* widget);
  gboolean (*widget_grab_focus)(GtkWidget*);
  void (*widget_set_halign)(GtkWidget*, GtkAlign);
  void (*widget_show_all)(GtkWidget*);
  GType (*window_get_type)(void);
  void (*window_set_geometry_hints)(GtkWindow*,
                                    GtkWidget*,
                                    GdkGeometry*,
                                    GdkWindowHints);
  void (*window_set_icon_name)(GtkWindow*, const gchar*);
} gtk;

#define gtk_box_get_type gtk.box_get_type
#define gtk_box_new gtk.box_new
#define gtk_box_pack_start gtk.box_pack_start
#define gtk_box_set_spacing gtk.box_set_spacing
#define gtk_container_add gtk.container_add
#define gtk_container_get_type gtk.container_get_type
#define gtk_container_set_border_width gtk.container_set_border_width
#define gtk_dialog_get_content_area gtk.dialog_get_content_area
#define gtk_dialog_get_type gtk.dialog_get_type
#define gtk_dialog_new_with_buttons gtk.dialog_new_with_buttons
#define gtk_dialog_run gtk.dialog_run
#define gtk_dialog_set_default_response gtk.dialog_set_default_response
#define gtk_entry_get_text gtk.entry_get_text
#define gtk_entry_get_type gtk.entry_get_type
#define gtk_entry_new gtk.entry_new
#define gtk_entry_set_activates_default gtk.entry_set_activates_default
#define gtk_entry_set_input_purpose gtk.entry_set_input_purpose
#define gtk_entry_set_placeholder_text gtk.entry_set_placeholder_text
#define gtk_entry_set_visibility gtk.entry_set_visibility
#define gtk_entry_set_width_chars gtk.entry_set_width_chars
#define gtk_events_pending gtk.events_pending
#define gtk_image_new_from_icon_name gtk.image_new_from_icon_name
#define gtk_init_check gtk.init_check
#define gtk_label_new gtk.label_new
#define gtk_main_iteration gtk.main_iteration
#define gtk_widget_destroy gtk.widget_destroy
#define gtk_widget_grab_focus gtk.widget_grab_focus
#define gtk_widget_set_halign gtk.widget_set_halign
#define gtk_widget_show_all gtk.widget_show_all
#define gtk_window_get_type gtk.window_get_type
#define gtk_window_set_geometry_hints gtk.window_set_geometry_hints
#define gtk_window_set_icon_name gtk.window_set_icon_name

static struct {
  void* handle;
  GTypeInstance* (*type_check_instance_cast)(GTypeInstance*, GType);
} g;

#define g_type_check_instance_cast g.type_check_instance_cast

//------------------------------------------------------------------------------
// Load GTK libraries and function pointers.
//------------------------------------------------------------------------------

static bool load_gtk(void) {
  if (gtk.handle != NULL) {
    return true;
  }

  // Load the gtk-3 library.
  gtk.handle = dlopen("libgtk-3.so", RTLD_LAZY);
  if (gtk.handle == NULL) {
    return false;
  }

  // Load the gobject-2.0 library.
  g.handle = dlopen("libgobject-2.0.so", RTLD_LAZY);
  if (g.handle == NULL) {
    dlclose(gtk.handle);
    gtk.handle = NULL;
    return false;
  }

  // Load function pointers.
  *(void**)(&gtk.box_get_type) = dlsym(gtk.handle, "gtk_box_get_type");
  *(void**)(&gtk.box_new) = dlsym(gtk.handle, "gtk_box_new");
  *(void**)(&gtk.box_pack_start) = dlsym(gtk.handle, "gtk_box_pack_start");
  *(void**)(&gtk.box_set_spacing) = dlsym(gtk.handle, "gtk_box_set_spacing");
  *(void**)(&gtk.container_add) = dlsym(gtk.handle, "gtk_container_add");
  *(void**)(&gtk.container_get_type) =
      dlsym(gtk.handle, "gtk_container_get_type");
  *(void**)(&gtk.container_set_border_width) =
      dlsym(gtk.handle, "gtk_container_set_border_width");
  *(void**)(&gtk.dialog_get_content_area) =
      dlsym(gtk.handle, "gtk_dialog_get_content_area");
  *(void**)(&gtk.dialog_get_type) = dlsym(gtk.handle, "gtk_dialog_get_type");
  *(void**)(&gtk.dialog_new_with_buttons) =
      dlsym(gtk.handle, "gtk_dialog_new_with_buttons");
  *(void**)(&gtk.dialog_run) = dlsym(gtk.handle, "gtk_dialog_run");
  *(void**)(&gtk.dialog_set_default_response) =
      dlsym(gtk.handle, "gtk_dialog_set_default_response");
  *(void**)(&gtk.entry_get_text) = dlsym(gtk.handle, "gtk_entry_get_text");
  *(void**)(&gtk.entry_get_type) = dlsym(gtk.handle, "gtk_entry_get_type");
  *(void**)(&gtk.entry_new) = dlsym(gtk.handle, "gtk_entry_new");
  *(void**)(&gtk.entry_set_activates_default) =
      dlsym(gtk.handle, "gtk_entry_set_activates_default");
  *(void**)(&gtk.entry_set_input_purpose) =
      dlsym(gtk.handle, "gtk_entry_set_input_purpose");
  *(void**)(&gtk.entry_set_placeholder_text) =
      dlsym(gtk.handle, "gtk_entry_set_placeholder_text");
  *(void**)(&gtk.entry_set_visibility) =
      dlsym(gtk.handle, "gtk_entry_set_visibility");
  *(void**)(&gtk.entry_set_width_chars) =
      dlsym(gtk.handle, "gtk_entry_set_width_chars");
  *(void**)(&gtk.events_pending) = dlsym(gtk.handle, "gtk_events_pending");
  *(void**)(&gtk.image_new_from_icon_name) =
      dlsym(gtk.handle, "gtk_image_new_from_icon_name");
  *(void**)(&gtk.init_check) = dlsym(gtk.handle, "gtk_init_check");
  *(void**)(&gtk.label_new) = dlsym(gtk.handle, "gtk_label_new");
  *(void**)(&gtk.main_iteration) = dlsym(gtk.handle, "gtk_main_iteration");
  *(void**)(&gtk.widget_destroy) = dlsym(gtk.handle, "gtk_widget_destroy");
  *(void**)(&gtk.widget_grab_focus) =
      dlsym(gtk.handle, "gtk_widget_grab_focus");
  *(void**)(&gtk.widget_set_halign) =
      dlsym(gtk.handle, "gtk_widget_set_halign");
  *(void**)(&gtk.widget_show_all) = dlsym(gtk.handle, "gtk_widget_show_all");
  *(void**)(&gtk.window_get_type) = dlsym(gtk.handle, "gtk_window_get_type");
  *(void**)(&gtk.window_set_geometry_hints) =
      dlsym(gtk.handle, "gtk_window_set_geometry_hints");
  *(void**)(&gtk.window_set_icon_name) =
      dlsym(gtk.handle, "gtk_window_set_icon_name");
  *(void**)(&g.type_check_instance_cast) =
      dlsym(g.handle, "g_type_check_instance_cast");

  // Check that all symbols could be loaded.
  if (gtk.box_get_type == NULL || gtk.box_new == NULL ||
      gtk.box_pack_start == NULL || gtk.box_set_spacing == NULL ||
      gtk.container_add == NULL || gtk.container_get_type == NULL ||
      gtk.container_set_border_width == NULL ||
      gtk.dialog_get_content_area == NULL || gtk.dialog_get_type == NULL ||
      gtk.dialog_new_with_buttons == NULL || gtk.dialog_run == NULL ||
      gtk.dialog_set_default_response == NULL || gtk.entry_get_text == NULL ||
      gtk.entry_get_type == NULL || gtk.entry_new == NULL ||
      gtk.entry_set_activates_default == NULL ||
      gtk.entry_set_input_purpose == NULL ||
      gtk.entry_set_placeholder_text == NULL ||
      gtk.entry_set_visibility == NULL || gtk.entry_set_width_chars == NULL ||
      gtk.events_pending == NULL || gtk.image_new_from_icon_name == NULL ||
      gtk.init_check == NULL || gtk.label_new == NULL ||
      gtk.main_iteration == NULL || gtk.widget_destroy == NULL ||
      gtk.widget_grab_focus == NULL || gtk.widget_set_halign == NULL ||
      gtk.widget_show_all == NULL || gtk.window_get_type == NULL ||
      gtk.window_set_geometry_hints == NULL ||
      gtk.window_set_icon_name == NULL || g.type_check_instance_cast == NULL) {
    dlclose(gtk.handle);
    dlclose(g.handle);
    gtk.handle = NULL;
    g.handle = NULL;
    return false;
  }

  return true;
}

//------------------------------------------------------------------------------
// GTK dialog function, called in a child process.
//------------------------------------------------------------------------------

static int get_from_gtk_dialog_subproc(const char* prompt,
                                       char* passphrase,
                                       size_t max_size) {
  // Init GTK.
  if ((!load_gtk()) || gtk_init_check(NULL, NULL) == FALSE) {
    passphrase[0] = 0;
    return PASSPHRASE_FAIL;
  }

  // Create the dialog.
  GtkWidget* dialog = gtk_dialog_new_with_buttons("ezcrypt",
                                                  NULL,
                                                  GTK_DIALOG_MODAL,
                                                  "_OK",
                                                  GTK_RESPONSE_OK,
                                                  "_Cancel",
                                                  GTK_RESPONSE_CANCEL,
                                                  NULL);
  GdkGeometry geometry;
  geometry.max_width = INT_MAX;
  geometry.max_height = -1;
  gtk_window_set_geometry_hints(
      GTK_WINDOW(dialog), dialog, &geometry, GDK_HINT_MAX_SIZE);
  gtk_window_set_icon_name(GTK_WINDOW(dialog), "dialog-password");
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
  gtk_container_set_border_width(GTK_CONTAINER(dialog), 20);
  GtkWidget* content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
  gtk_box_set_spacing(GTK_BOX(content_area), 20);

  // Add the prompt (icon + label).
  GtkWidget* hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 10);
  GtkWidget* image =
      gtk_image_new_from_icon_name("dialog-password", GTK_ICON_SIZE_DIALOG);
  GtkWidget* label = gtk_label_new(prompt);
  gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(content_area), hbox);

  // Add the passphrase entry.
  // TODO: gtk_password_entry_new() in GTK 4.
  GtkWidget* entry = gtk_entry_new();
  gtk_widget_set_halign(entry, GTK_ALIGN_FILL);
  gtk_entry_set_placeholder_text(GTK_ENTRY(entry), "Passphrase");
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
  gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
  gtk_entry_set_input_purpose(GTK_ENTRY(entry), GTK_INPUT_PURPOSE_PASSWORD);
  gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
  gtk_widget_grab_focus(entry);
  gtk_container_add(GTK_CONTAINER(content_area), entry);

  gtk_widget_show_all(dialog);

  // Show the dialog and wait for the response.
  int result = PASSPHRASE_FAIL;
  int dialog_res = gtk_dialog_run(GTK_DIALOG(dialog));
  if (dialog_res == GTK_RESPONSE_OK) {
    const char* text = gtk_entry_get_text(GTK_ENTRY(entry));
    if (string_copy(text, passphrase, max_size)) {
      result = PASSPHRASE_OK;
    } else {
      fprintf(stderr, "***Error: Too long passphrase.\n");
      result = PASSPHRASE_FAIL;
    }
  } else {
    result = PASSPHRASE_ABORT;
  }

  if (dialog) {
    gtk_widget_destroy(dialog);
  }

  // Drain all GTK events (required for actually closing the dialog).
  while (gtk_events_pending()) {
    gtk_main_iteration();
  }

  return result;
}

//------------------------------------------------------------------------------
// Public functions.
//------------------------------------------------------------------------------

static pid_t s_pid;

int get_passphrase_from_gtk_dialog(const char* prompt,
                                   char* passphrase,
                                   size_t max_size) {
  passphrase[0] = 0;

  // Set up redirection pipes.
  int pipefd[2];
  if (pipe(pipefd) == -1) {
    fprintf(stderr, "***Warning: Passphrase pipe creation failed.\n");
    return PASSPHRASE_FAIL;
  }

  // Run the dialog in a child process.
  s_pid = fork();
  if (s_pid < 0) {
    fprintf(stderr, "***Warning: Passphrase dialog process creation failed.\n");
    return PASSPHRASE_FAIL;
  }
  if (s_pid > 0) {
    // We're in the parent process.
    // Close unused write end of the pipe.
    close(pipefd[1]);

    // Read the passphrase from the child process.
    size_t total_bytes_read = 0;
    while ((total_bytes_read + 1U) < max_size) {
      const size_t max_bytes_to_read = max_size - 1U - total_bytes_read;
      const ssize_t bytes_read =
          read(pipefd[0], &passphrase[total_bytes_read], max_bytes_to_read);
      if (bytes_read < 0) {
        fprintf(stderr, "***Warning: Passphrase dialog pipe broken.\n");
        return PASSPHRASE_FAIL;
      }
      if (bytes_read == 0) {
        break;
      }
      total_bytes_read += (size_t)bytes_read;
    }
    passphrase[total_bytes_read] = 0;

    // Wait for the child process to finish.
    int status = 0;
    int waitpid_result = waitpid(s_pid, &status, 0);
    s_pid = 0;
    if (waitpid_result == -1 || !WIFEXITED(status)) {
      fprintf(stderr,
              "***Warning: Passphrase dialog process died unexpectedly.\n");
      return PASSPHRASE_FAIL;
    }

    // Get the return value from the child process.
    const int result = WEXITSTATUS(status);
    if (result < PASSPHRASE_FAIL || result > PASSPHRASE_OK) {
      fprintf(stderr,
              "***Warning: Unexpected dialog process exit status: %d\n",
              result);
      return PASSPHRASE_FAIL;
    }
    return result;
  }

  // We're in the child process.
  // Close unused read end of the pipe.
  close(pipefd[0]);

  // Use the default locale (this silences warnings on some systems).
  (void)setenv("LANG", "C", 1);

  // Show the dialog.
  int passphrase_status =
      get_from_gtk_dialog_subproc(prompt, passphrase, max_size);

  // Write the passphrase to the pipe.
  size_t bytes_to_write = strlen(passphrase);
  size_t pos = 0;
  while (pos < bytes_to_write) {
    const ssize_t bytes_written =
        write(pipefd[1], &passphrase[pos], bytes_to_write);
    if (bytes_written < 0) {
      passphrase_status = PASSPHRASE_FAIL;
      break;
    }
    pos += bytes_written;
    bytes_to_write -= (size_t)bytes_written;
  }

  // Closing the pipe will send EOF.
  close(pipefd[1]);

  // Exit the process with the function result.
  exit(passphrase_status);
}

void kill_passphrase_child_processes_gtk(void) {
  if (s_pid > 0) {
    (void)kill(s_pid, SIGTERM);
    s_pid = 0;
  }
}
