// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "../crypto/aes.h"
#include "minitest.h"

#include <stdint.h>
#include <stdlib.h>

// Key and IV to use in the benchmarks.
static const uint8_t KEY[32] = {0x60, 0x3d, 0xeb, 0x10, 0x15, 0xca, 0x71, 0xbe,
                                0x2b, 0x73, 0xae, 0xf0, 0x85, 0x7d, 0x77, 0x81,
                                0x1f, 0x35, 0x2c, 0x07, 0x3b, 0x61, 0x08, 0xd7,
                                0x2d, 0x98, 0x10, 0xa3, 0x09, 0x14, 0xdf, 0xf4};
static const uint8_t IV[16] = {0x00,
                               0x01,
                               0x02,
                               0x03,
                               0x04,
                               0x05,
                               0x06,
                               0x07,
                               0x08,
                               0x09,
                               0x0a,
                               0x0b,
                               0x0c,
                               0x0d,
                               0x0e,
                               0x0f};

BENCH_MAIN("AES-CBC-256 cipher") {
  BENCH_CASE("Encrypt - 64 byte buffer, 1024 times") {
    // Setup.
    const size_t buf_size = 64;
    uint8_t* buf = calloc(buf_size, 1);
    aes256_ctx_t ctx;
    aes_256_init(&ctx, KEY, IV);

    // Benchmark loop.
    BENCH_LOOP_BEGIN(100);
    for (int i = 0; i < 1024; ++i) {
      aes_256_cbc_encrypt_buffer(&ctx, buf, buf_size);
    }
    BENCH_LOOP_END();

    // Teardown.
    free(buf);
  }

  BENCH_CASE("Decrypt - 64 byte buffer, 1024 times") {
    // Setup.
    const size_t buf_size = 64;
    uint8_t* buf = calloc(buf_size, 1);
    aes256_ctx_t ctx;
    aes_256_init(&ctx, KEY, IV);

    // Benchmark loop.
    BENCH_LOOP_BEGIN(100);
    for (int i = 0; i < 1024; ++i) {
      aes_256_cbc_decrypt_buffer(&ctx, buf, buf_size);
    }
    BENCH_LOOP_END();

    // Teardown.
    free(buf);
  }

  BENCH_CASE("Encrypt - 64KiB buffer") {
    // Setup.
    const size_t buf_size = 65536U;
    uint8_t* buf = calloc(buf_size, 1);
    aes256_ctx_t ctx;
    aes_256_init(&ctx, KEY, IV);

    // Benchmark loop.
    BENCH_LOOP_BEGIN(100);
    aes_256_cbc_encrypt_buffer(&ctx, buf, buf_size);
    BENCH_LOOP_END();

    // Teardown.
    free(buf);
  }

  BENCH_CASE("Decrypt - 64KiB buffer") {
    // Setup.
    const size_t buf_size = 65536U;
    uint8_t* buf = calloc(buf_size, 1);
    aes256_ctx_t ctx;
    aes_256_init(&ctx, KEY, IV);

    // Benchmark loop.
    BENCH_LOOP_BEGIN(100);
    aes_256_cbc_decrypt_buffer(&ctx, buf, buf_size);
    BENCH_LOOP_END();

    // Teardown.
    free(buf);
  }
}
