// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "../crypto/poly1305.h"
#include "minitest.h"

#include <stdint.h>
#include <stdlib.h>

// Key to use in the benchmarks.
static const uint8_t KEY[POLY1305_KEY_BYTES] = {
    0x85, 0xd6, 0xbe, 0x78, 0x57, 0x55, 0x6d, 0x33, 0x7f, 0x44, 0x52,
    0xfe, 0x42, 0xd5, 0x06, 0xa8, 0x01, 0x03, 0x80, 0x8a, 0xfb, 0x0d,
    0xb2, 0xfd, 0x4a, 0xbf, 0xf6, 0xaf, 0x41, 0x49, 0xf5, 0x1b};

BENCH_MAIN("Poly1305 keyed hash function") {
  BENCH_CASE("64 byte buffer, 1024 times") {
    // Setup.
    const size_t buf_size = 64;
    uint8_t* buf = calloc(buf_size, 1);
    poly1305_ctx_t ctx;
    poly1305_init(&ctx, KEY);

    // Benchmark loop.
    BENCH_LOOP_BEGIN(100);
    for (int i = 0; i < 1024; ++i) {
      poly1305_append(&ctx, buf, buf_size);
    }
    BENCH_LOOP_END();

    // Teardown.
    free(buf);
  }

  BENCH_CASE("64KiB buffer") {
    // Setup.
    const size_t buf_size = 65536U;
    uint8_t* buf = calloc(buf_size, 1);
    poly1305_ctx_t ctx;
    poly1305_init(&ctx, KEY);

    // Benchmark loop.
    BENCH_LOOP_BEGIN(100);
    poly1305_append(&ctx, buf, buf_size);
    BENCH_LOOP_END();

    // Teardown.
    free(buf);
  }
}
