// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "../crypto/zkdf.h"
#include "minitest.h"

#include <stdint.h>

// Salt and passphrase to use in the benchmarks.
static const uint8_t SALT[ZKDF_SALT_BYTES] = {
    0xa2, 0x6a, 0xa0, 0x06, 0x50, 0x19, 0x53, 0x57, 0xe1, 0xa1, 0xd0,
    0xe7, 0x24, 0x44, 0x11, 0xc0, 0x5a, 0x8d, 0x99, 0x10, 0xbc, 0xbf,
    0x81, 0xc6, 0x41, 0xe9, 0xcb, 0xdc, 0xeb, 0x7b, 0x26, 0x57};

static const char* PASS = "Hello world please keep this secret secret";

BENCH_MAIN("ZKDF key derivation algorithm") {
  BENCH_CASE("Derive key - 1 pass") {
    BENCH_LOOP_BEGIN(100);
    uint8_t key[ZKDF_KEY_BYTES];
    zkdf_make_key(PASS, SALT, 1, key);
    BENCH_LOOP_END();
  }

  BENCH_CASE("Derive key - 10 passes") {
    BENCH_LOOP_BEGIN(20);
    uint8_t key[ZKDF_KEY_BYTES];
    zkdf_make_key(PASS, SALT, 10, key);
    BENCH_LOOP_END();
  }

  BENCH_CASE("Derive key - 100 passes") {
    BENCH_LOOP_BEGIN(2);
    uint8_t key[ZKDF_KEY_BYTES];
    zkdf_make_key(PASS, SALT, 100, key);
    BENCH_LOOP_END();
  }
}
