// -*- mode: c; tab-width: 2; indent-tabs-mode: nil; -*-
//------------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
//
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
//
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
//
// For more information, please refer to <https://unlicense.org/>
//------------------------------------------------------------------------------

#include "../crypto/xoshiro.h"
#include "minitest.h"

#include <stdint.h>

TEST_MAIN("xoshiro - PRNG") {
  TEST_CASE("Initialize state") {
    // Given: A seed vector.
    static const uint8_t seed[XOSHIRO256_BYTES] = {
        1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12, 13, 14, 15, 16,
        17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32};

    // When: Initializing the context.
    xoshiro256_state_t state;
    xoshiro256_init(seed, &state);

    // Then: The state should be correct.
    CHECK_EQ_UINTX(state.s[0], 0x0807060504030201U);
    CHECK_EQ_UINTX(state.s[1], 0x100f0e0d0c0b0a09U);
    CHECK_EQ_UINTX(state.s[2], 0x1817161514131211U);
    CHECK_EQ_UINTX(state.s[3], 0x201f1e1d1c1b1a19U);
  }

  TEST_CASE("Generate random numbers") {
    // Given: An initialized state.
    static const uint8_t seed[XOSHIRO256_BYTES] = {
        1,  2,  3,  4,  5,  6,  7,  8,  9,  10, 11, 12, 13, 14, 15, 16,
        17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32};
    xoshiro256_state_t state;
    xoshiro256_init(seed, &state);

    // When: Generating eight consecutive numbers.
    const uint64_t x1 = xoshiro256ss(&state);
    const uint64_t x2 = xoshiro256ss(&state);
    const uint64_t x3 = xoshiro256ss(&state);
    const uint64_t x4 = xoshiro256ss(&state);
    const uint64_t x5 = xoshiro256ss(&state);
    const uint64_t x6 = xoshiro256ss(&state);
    const uint64_t x7 = xoshiro256ss(&state);
    const uint64_t x8 = xoshiro256ss(&state);

    // Then: The numbers should be correct.
    CHECK_EQ_UINTX(x1, 0x52bc258ef861cbe8U);
    CHECK_EQ_UINTX(x2, 0xbc258ef861cb3280U);
    CHECK_EQ_UINTX(x3, 0x2d013c0ee31e1c12U);
    CHECK_EQ_UINTX(x4, 0x2506e98014e9ac07U);
    CHECK_EQ_UINTX(x5, 0x62e739f7970c2caeU);
    CHECK_EQ_UINTX(x6, 0x6b67d55e63fe394dU);
    CHECK_EQ_UINTX(x7, 0xf0322ec27ddcb89eU);
    CHECK_EQ_UINTX(x8, 0x511d7d79121616b3U);
  }
}
